<?php
define('ASTRO_DB_HOST', 'localhost');
define('ASTRO_DB_USER', 'francoist_modx');
define('ASTRO_DB_PASS', 'mr?PH@(}90,O');
define('ASTRO_DB_NAME', 'francoist_modx');

error_log("reached here!");

class Webcamp {
	const OPT_SOCKET_TIMEOUT = 20;
	const OPT_CACHE_TYPE = WSDL_CACHE_NONE;
	const OPT_DISPLAY_TRACE = 1;
	const OPT_THROW_EXCEPTIONS = 1;

	private $db;

	private $_user;
	private $_pass;
	private $_lang;
	private $operations = array();
	private $wsdl = array(
		'WBC' => 'https://www.webcamp.fr/WebCampServices/?wsdl',
		'PRV' => 'https://beta.webcamp.fr/PrivateServices/PrivateServicesBean?wsdl'
	);

	public function __construct($user = null, $pass = null, $lang = 'fr', $wsdl = '') {
		ini_set('default_socket_timeout', 20);
		ini_set('soap.wsdl_cache_dir', './wsdl_cache');

		$this->db = new PDO('mysql:host='.ASTRO_DB_HOST.';dbname='.ASTRO_DB_NAME, ASTRO_DB_USER, ASTRO_DB_PASS);

		$this->_user = $user;
		$this->_pass = $pass;
		$this->_lang = $lang;

		$wsdl = !isset($this->wsdl[$wsdl]) ? $this->wsdl['WBC'] : $this->wsdl[$wsdl];

		$options = array (
			'connection_timeout' => self::OPT_SOCKET_TIMEOUT,
			'location' => $wsdl,
			'cache_wsdl' => self::OPT_CACHE_TYPE,
			'uri' => 'http://webservices.webcamp.fr',
			'trace' => self::OPT_DISPLAY_TRACE,
			'exceptions ' => self::OPT_THROW_EXCEPTIONS,
			'soap_version' => SOAP_1_2,
			'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP
		);

		$this->client = new SoapClient($wsdl, $options);

		foreach ($this->client->__getFunctions() as $f) {
			preg_match('~\w+\s([^\(]+)\(~', $f, $m);
			$this->operations[] = $m[1];
		}
	}

	public function randomString($length = 12) {
		return substr(str_shuffle(str_repeat($x = '0123456789abcdefghijklmnopqrstuvwxyz', ceil($length/strlen($x)))), 1, $length);
	}

	public function array_encode($array) {
		return base64_encode(json_encode($array));
	}

	public function array_decode($array) {
		return json_decode(base64_decode($array), true);
	}

	public function array_decode_json($array) {
		return base64_decode($array);
	}

	/**
	 *	Retourne le contexte non-traité
	 *	@param int $centre
	 *	@return array
	 */
	public function returnContext($centre) {

		$request = array (
			'idUser' => WEBCAMP_USER,
			'password' => WEBCAMP_PASS,
			'language' => $this->_lang,
			'idSite' => $centre,
			'returnSiteOnly' => 0,
			'returnProduct' => 1,
			'returnMediaObj' => 1
		);

		$response = json_decode(json_encode($this->client->cpg_Context($request)), true);
		return $response;
	}

	/**
	 *	Return availabilities
	 *	@param int $centre
	 *	@param string $_arrival
	 *	@param string $_departure
	 *	@param int $people
	 *	@param string $type
	 *	@param string $productId
	 *	@return array
	 */
	public function returnAvailabilities($centre, $_arrival, $_departure, $people = 0, $type = '', $productId = '', $debug = false) {
		global $modx;

		$arrival = new DateTime($_arrival);
		$departure = new DateTime($_departure);
		$duration = sprintf('P%sD', ($departure->format('U') - $arrival->format('U')) / 86400);
		$searchBothTypes = true;

		// LOCATION
		$request = array (
			'idUser' => WEBCAMP_USER,
			'password' => WEBCAMP_PASS,
			'language' => 'fr',
			'arrival' => $arrival->format('c'),
			'duration' => $duration,
			'person' => $people,
			//'searchPlace' => array(
			//	'idSite' => $centre,
			//),
			'searchType' => 4,
			'returnProduct' => 0,
			'returnMediaObj' => 0,
		);

		if(!empty($centre)) {
			$request['searchPlace'] = array(
				'idSite' => $centre,
			);
		}

		if(!empty($type) and !in_array($type, ['camping', 'location'])) {
			$request['productType'] = $type;
			$searchBothTypes = false;
		}

		if(!empty($productId)) {
			$_product = explode(',', $productId);
			$_id1 = $_product[0];
			$_idProduct = $_product[1];

			$request['searchPlace'] = array(
				'idSite' => $centre,
				'idProduct' => array(
					'id1' => $_id1,
					'idProduct' => $_idProduct,
				),
			);
		}

		// if($searchBothTypes) {
		// 	error_log('TRUE');
		// }
		// else {
		// 	error_log('FALSE');
		// }

		if($searchBothTypes) {
			$request['productType'] = 'camping';
			$response_c = json_decode(json_encode($this->client->cpg_Availability($request)), true);

			$request['productType'] = 'location';
			$response = json_decode(json_encode($this->client->cpg_Availability($request)), true);
		}
		else {
			$response = json_decode(json_encode($this->client->cpg_Availability($request)), true);
		}

		// one or X values
		if(isset($response['stay']['idStay'])) {
			$response['stay'] = array($response['stay']);
		}

		if($searchBothTypes) {
			if(isset($response_c['stay']['idStay'])) {
				$response_c['stay'] = array($response_c['stay']);
			}

			foreach($response_c['stay'] as $row) {
				$response['stay'][] = $row;
			}
		}

		// Debug? Return response
		if($debug) {
			return $response;
		}

		$stay = [];
		foreach($response['stay'] as $res) {
			$product = sprintf('%s,%s', $res['idStay']['idProduct']['id1'], $res['idStay']['idProduct']['idProduct']);

			if(!empty($productId)) {
				if($product !== $productId) continue;
			}

			$the_token = $this->randomString(12);
			$stay[$the_token] = $res;

			$path = sprintf('%s/stay/%s.json', __DIR__, $the_token);
			$res['image'] = $modx->runSnippet('returnFirstImageWebcamp', ['thelisId' => $product]);

			$the_stay = $this->array_encode($res);
			$the_date = new DateTime();
			$fileContent = [
				'stay' => $the_stay,
				'date' => $the_date->format('c'),
			];

			$file = fopen($path, 'w');
			fwrite($file, json_encode($fileContent));
			fclose($file);
		}

		return $stay;
	}

	/**
	 *	Get stay data from token
	 *	@param string $token
	 *	@param bool $json
	 *	@return string
	 */
	public function getStayData($token, $json = false) {
		$path = sprintf('%s/stay/%s.json', __DIR__, $token);
		if(!file_exists($path)) return false;

		$fileContent = json_decode(file_get_contents($path), true);
		$output = $fileContent['stay'];

		if($json) {
			return $this->array_decode_json($output);
		}
		else {
			return $this->array_decode($output);
		}
	}


	/**
	 *	Retourne les suppléments à partir d'un token
	 *	@param int $centre
	 *	@param string $token
	 *	@return array
	 */
	public function returnSupplementsFromToken($centre, $token) {
		$data = $this->getStayData($token, false);
		$id1 = $data['idStay']['idProduct']['id1'];
		$idProduct = $data['idStay']['idProduct']['idProduct'];
		return $this->returnSupplements($centre, $id1, $idProduct);
	}


	/**
	 *
	 *	Retourne les suppléments d'un produit
	 *
	 */
	public function returnSupplements($centre, $id1, $idProduct) {

		$request = array (
			'idUser' => WEBCAMP_USER,
			'password' => WEBCAMP_PASS,
			'language' => $this->_lang,
			'idSite' => $centre,
			'returnSiteOnly' => 0,
			'returnProduct' => 1,
			'returnMediaObj' => 1
		);

		$response = json_decode(json_encode($this->client->cpg_Context($request)), true);

		$supplements = [];
		foreach($response['tariffProduct'] as $product) {
			$productId = sprintf('%s,%s', $product['idTariff']['id1'], $product['idTariff']['idTariffProduct']);
			$searchId = sprintf('%s,%s', $id1, $idProduct);
			if($productId !== $searchId) continue;

			$productSupplements = [];
			foreach($product['supplement'] as $supplement) {
				$supplement['name'] = strip_tags(nl2br($supplement['name']));
				$supplement['description'] = strip_tags(nl2br($supplement['description']));
				$productSupplements[$supplement['idSupp']] = $supplement;
			}

			$supplements = $productSupplements;
		}

		asort($supplements);
		return $supplements;
	}

	/**
	 *
	 *	Retourne le prix unitaire d'un séjour
	 *
	 */
	public function returnUnitPrice($centre, $data) {
		$data = json_decode($data, true);

		$request = array (
			'idUser' => WEBCAMP_USER,
			'password' => WEBCAMP_PASS,
			'language' => $this->_lang,
		);

		foreach($data as $key => $row) {
			if($key == 'supplement') {
				$new = [];
				foreach($row as $supp) {
					if($supp['number'] !== 0 && !empty($supp['number'])) $new[] = $supp;
				}
				$row = $new;
			}
			$request[$key] = $row;
		}

		$response = json_decode(json_encode($this->client->cpg_UnitPrice($request)), true);
		return $response;
	}


	/**
	 *
	 *	Retourne le prix d'un séjour
	 *
	 */
	public function returnPrice($centre, $data) {
		$data = json_decode($data, true);

		$request = array (
			'idUser' => WEBCAMP_USER,
			'password' => WEBCAMP_PASS,
			'language' => $this->_lang,
		);

		foreach($data as $key => $row) {
			if($key == 'supplement') {
				$new = [];
				foreach($row as $supp) {
					if($supp['number'] !== 0 && !empty($supp['number'])) $new[] = $supp;
				}
				$row = $new;
			}
			$request[$key] = $row;
		}

		$response = json_decode(json_encode($this->client->cpg_Price($request)), true);

		return $response;
	}

	/**
	 *	Retourne les modes de paiement
	 *	@param int $centre
	 *	@return array
	 */
	public function returnPaymentMethods($centre) {

		$request = array (
			'idUser' => WEBCAMP_USER,
			'password' => WEBCAMP_PASS,
			'language' => $this->_lang,
			'idSite' => $centre,
			'returnSiteOnly' => 0,
			'returnProduct' => 1,
			'returnMediaObj' => 1
		);

		$response = json_decode(json_encode($this->client->cpg_Context($request)), true);
		return $response['camping']['payment'];
	}

	/**
	 *
	 *	Retourne la liste des champs pour les coordonnées client
	 *
	 */
	public function returnClientInformationFields($centre) {

		$request = array (
			'idUser' => WEBCAMP_USER,
			'password' => WEBCAMP_PASS,
			'language' => $this->_lang,
			'idSite' => 6647,
			'returnSiteOnly' => 0,
			'returnProduct' => 1,
			'returnMediaObj' => 1
		);

		$response = json_decode(json_encode($this->client->cpg_Context($request)), true);

		$fields = [];
		foreach($response['camping']['customerListInformation'] as $key => $value) {
			// if($type)

			$type = 'text';
		}
	}

	/**
	 *	Passe une commande à Thelis
	 *	@param int $centre
	 *	@param string $data
	 *	@return array
	 */
	public function processOrder($centre, $data) {
		$data = json_decode($data, true);

		$request = array (
			'idUser' => WEBCAMP_USER,
			'password' => WEBCAMP_PASS,
			'language' => 'fr',
		);

		foreach($data as $key => $row) {
			$case = $row;

			// Supplements
			if($key == 'supplement') {
				$supplements = [];
				foreach($row as $supp) {
					$supplements[] = $supp;
				}
				$case = $supplements;
			}

			$request[$key] = $case;
		}

		$response = json_decode(json_encode($this->client->cpg_Order($request)), true);

		$logOrder = $this->logOrder($request, $response, $response['idOrder'] ?: '');

		if(isset($response['idOrder']) and isset($response['idPayment'])) {
			$logPayment = $this->logPayment($response['idOrder'], $response['idPayment'], floatval($data['deposit']));
		}

		return $response;
	}

	/**
	 *	Logger les réservations
	 *	@param array $request
	 *	@param array $response
	 *	@return int
	 */
	private function logOrder($request, $response, $order = '') {

		if(isset($request['payment']['paymentCardType']['cardNumber'])) {
			$request['payment']['paymentCardType']['cardNumber'] = str_repeat('*', strlen($request['payment']['paymentCardType']['cardNumber']));
		}

		if(isset($request['payment']['paymentCardType']['expireDate'])) {
			$request['payment']['paymentCardType']['expireDate'] = str_repeat('*', strlen($request['payment']['paymentCardType']['expireDate']));
		}

		if(isset($request['payment']['paymentCardType']['seriesCode'])) {
			$request['payment']['paymentCardType']['seriesCode'] = str_repeat('*', strlen($request['payment']['paymentCardType']['seriesCode']));
		}

		$_request = $this->db->quote($this->array_encode($request));
		$_response = $this->db->quote($this->array_encode($response));

		if(empty($order)) $_order = NULL;
		else $_order = $this->db->quote($order);

		$this->db->query("
			INSERT INTO webcamp_reservations
			(order_number, request, response)
			VALUES ($_order, $_request, $_response)
			");

		return $this->db->lastInsertId();
	}

	/**
	 *	Logger un paiement
	 *	@param array $order
	 *	@param array $payment
	 *	@param float $amount
	 *	@return int
	 */
	private function logPayment($order, $payment, $amount = 0) {
		$_order = $this->db->quote($order);
		$_payment = $this->db->quote($payment);
		$_amount = floatval($amount);

		$this->db->query("
			INSERT INTO webcamp_payments
			(order_number, payment_number, amount)
			VALUES ($_order, $_payment, $_amount)
			");

		return $this->db->lastInsertId();
	}

	/**
	 *	Obtenir le statut d'une réservation
	 *	@param string $order
	 *	@return array
	 */
	public function orderStatus($order) {

		$request = array(
			'idOrder' => $order,
			'idUser' => $this->_user,
			'password' => $this->_pass,
			'language' => $this->_lang,
		);

		$response = json_decode(json_encode($this->client->cpg_OrderStatus($request)), true);
		if($response['error'] !== 0) return false;
		
		return $response;
	}

	/**
	 *	Payer le solde
	 *	@param string $order
	 *	@return array
	 */
	public function processPayment($order, $data) {

		// Paiement par carte
		if($data['method'] == 'card') {

			// Mode de paiement du DOMAINE LABORDE (via cpg_Context)
			$idPaymentType = array(
				'id1' => 9715,
				'idPaymentType' => 3,
			);

			// Données du paiement dans la variable $data
			$paymentCardType = array(
				'cardNumber' => $data['cardnumber'],
				'cardType' => $data['cardtype'] ?: 'MC',
				'expireDate' => date('c', strtotime('01/'.$data['cardexpire'])),
				'seriesCode' => $data['cardcvv'],
			);

			$request = array(
				'idOrder' => $order,
				'idPaymentType' => $idPaymentType,
				'payment' => array(
					'holderName' => $data['cardname'],
					'paymentCardType' => $paymentCardType,
				),
				'paymentType' => 1,
				'value' => floatval($data['amount']),
				'idUser' => $this->_user,
				'password' => $this->_pass,
			);
		}

		else if($data['method'] == 'transfer') {

			// Mode de paiement du DOMAINE LABORDE (via cpg_Context)
			$idPaymentType = array(
				'id1' => 22756,
				'idPaymentType' => 6,
			);

			$directBill = $order;

			$request = array(
				'idOrder' => $order,
				'idPaymentType' => $idPaymentType,
				'payment' => array(
					'holderName' => $data['cardname'],
					'directBill' => $directBill,
				),
				'paymentType' => 1,
				'value' => floatval($data['amount']),
				'idUser' => $this->_user,
				'password' => $this->_pass,
			);
		}

		else {
			return false;
		}

		$response = json_decode(json_encode($this->client->cpg_Payment($request)), true);

		if($response['error'] !== 0) return false;

		$logPayment = $this->logPayment($order, $response['idPayment'], floatval($data['amount']));

		return $response;
	}

	/**
	 *	Obtenir le status d'un paiement
	 *	@param string $payment
	 *	@return array
	 */
	public function returnPayment($payment, $order = '') {

		$request = array(
			'idOrder' => $order,
			'idPayment' => $payment,
			'idUser' => $this->_user,
			'password' => $this->_pass,
			'language' => $this->_lang,
		);

		$response = json_decode(json_encode($this->client->cpg_PaymentStatus($request)), true);

		return $response;
	}

	/**
	 *	Logger un paiement
	 *	@param array $order
	 *	@param array $payment
	 *	@return int
	 */
	public function returnPayments($order) {
		$_order = $this->db->quote($order);

		$payments = $this->db->query("
			SELECT * FROM webcamp_payments
			WHERE order_number = $_order
			");

		if($payments == false) return false;

		$response = [];
		foreach($payments as $row) {
			$_payment = $this->returnPayment($row['payment_number'], $row['order_number']);
			$_payment['number'] = $row['payment_number'];
			$_payment['amount'] = floatval($row['amount']);
			$_payment['date'] = $row['creation'];
			$response[] = $_payment;
		}

		return $response;
	}
}
