<?php

class WebcampBeta extends Webcamp {
	const OPT_SOCKET_TIMEOUT = 20;
	const OPT_CACHE_TYPE = WSDL_CACHE_NONE;
	const OPT_DISPLAY_TRACE = 1;
	const OPT_THROW_EXCEPTIONS = 1;

	private $_user;
	private $_pass;
	private $_lang;
	private $operations = array();
	private $wsdl = array(
		'WBC' => 'https://www.webcamp.fr/WebCampServices/?wsdl',
		'PRV' => 'https://beta.webcamp.fr/PrivateServices/PrivateServicesBean?wsdl'
	);

	public function __construct($user, $pass, $lang = 'fr', $wsdl = '') {
		ini_set('default_socket_timeout', 20);
		ini_set('soap.wsdl_cache_dir', './wsdl_cache');

		$this->_user = $user;
		$this->_pass = $pass;
		$this->_lang = $lang;

		$wsdl = !isset($this->wsdl[$wsdl]) ? $this->wsdl['WBC'] : $this->wsdl[$wsdl];

		$options = array (
			'connection_timeout' => self::OPT_SOCKET_TIMEOUT,
			'location' => $wsdl,
			'cache_wsdl' => self::OPT_CACHE_TYPE,
			'uri' => 'http://webservices.webcamp.fr',
			'trace' => self::OPT_DISPLAY_TRACE,
			'exceptions ' => self::OPT_THROW_EXCEPTIONS,
			'soap_version' => SOAP_1_2,
			'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP
		);

		$this->client = new SoapClient($wsdl, $options);

		foreach ($this->client->__getFunctions() as $f) {
			preg_match('~\w+\s([^\(]+)\(~', $f, $m);
			$this->operations[] = $m[1];
		}
	}

	/**
	 *	Passe une commande à Thelis
	 *	@param int $centre USELESS
	 *	@param string $data USELESS
	 *	@return array
	 */
	public function processOrder($centre = '', $data = '') {
		$data = json_decode($data, true);

		$request = array (
			'idUser' => WEBCAMP_USER,
			'password' => WEBCAMP_PASS,
			'language' => 'fr',
			'comment' => 'Réservation Bêta Astropof',
			'customer' => array(
				'civility' => 'Mr.',
				'lastname' => 'St-Laurent',
				'firstname' => 'Jacob',
				'birthday' => '1999-01-01T00:00:00',
				'address' => '123 rue Beta',
				'zipCode' => '1234',
				'city' => 'Montréal',
				'country' => 'CA',
				'email' => 'jacob@astropof.com',
				'phone' => '4384034142',
			),
			'deposit' => 87.68,
			'expenseReservation' => 0.0,
			'idPaymentType' => array(
				'id1' => 9715,
				'idPaymentType' => 2,
			),
			'idStay' => array(
				'beginDate' => '2020-07-24T00:00:00.000+02:00',
				'endDate' => '2020-07-26T00:00:00.000+02:00',
				'idProduct' => array(
					'id1' => 9715,
					'idProduct' => 4,
				),
				'id2' => 88,
				'idTariff' => array(
					'id1' => 9715,
					'idTariffProduct' => 4,
				),
			),
			'cancelEnable' => false,
			'payment' => array(
				'holderName' => 'Jacob St-Laurent',
				'directBill' => 'Test Astropof',
			),
			'price' => 87.2,
			'supplement' => array(
				array(
					'idSupp' => 1,
					'number' => 1,
				),
			),
		);

		$response = json_decode(json_encode($this->client->cpg_Order($request)), true);

		return $response;
	}

	/**
	 *	Obtenir le status d'une réservation
	 *	@param string $order
	 *	@return array
	 */
	public function orderStatus($order) {

		$request = array(
			'idOrder' => $order,
			'idUser' => $this->_user,
			'password' => $this->_pass,
			'language' => $this->_lang,
		);

		$response = json_decode(json_encode($this->client->cpg_OrderStatus($request)), true);

		return $response;
	}

	/**
	 *	Obtenir le status d'un paiement
	 *	@param string $payment
	 *	@return array
	 */
	public function returnPayment($payment, $order = '') {

		$request = array(
			'idOrder' => $order,
			'idPayment' => $payment,
			'idUser' => $this->_user,
			'password' => $this->_pass,
			'language' => $this->_lang,
		);

		$response = json_decode(json_encode($this->client->cpg_PaymentStatus($request)), true);

		return $response;
	}
}