const tunnelIdentifier = "#quoteTunnel";
const theUrl = "https://www.domainelaborde.com/fr/";
const assetsUrl = "https://www.domainelaborde.com/assets/";

// Get the language
function tunnel_getLanguage() {
  return $(tunnelIdentifier).data("lang") ?? "fr";
}

// Get query param
function _get(param, defaultValue = "") {
  const vars = {};
  window.location.href.replace(
    /[?&]+([^=&]+)=([^&]*)/gi,
    function (m, key, value) {
      vars[key] = value;
    }
  );

  const urlparam = defaultValue;
  if (window.location.href.indexOf(param) > -1) {
    urlparam = vars[param];
  }
  return urlparam;
}

// Get stay data
function tunnel_getStayData() {
  var result = "";
  $.ajax({
    method: "GET",
    url: assetsUrl + "tunnel/ajax/stay.php",
    data: {
      lang: tunnel_getLanguage(),
      token: _get("token"),
    },
    async: false,
    success: function (data) {
      result = data;
    },
  });

  try {
    result = JSON.parse(result);
  } catch (e) {
    result = new Object();
  }

  return result;
}

// Define if loaded or not
function tunnel_loaded(loaded = false) {
  const tunnel = $(tunnelIdentifier);
  var div = $(".tunnel-loading", tunnel).first();

  if (loaded) {
    div.addClass("loaded");
  } else {
    div.removeClass("loaded");
  }
}

// Toggle the sidebar
function tunnel_toggleSidebar() {
  const tunnel = $(tunnelIdentifier);
  var sidebar = $(".tunnel-sidebar", tunnel);
  var button = $(".sidebar-button", tunnel).first();
  sidebar.toggleClass("open");

  if (sidebar.hasClass("open")) {
    $("i", button).first().removeClass("fa-list");
    $("i", button).first().addClass("fa-times");
  } else {
    $("i", button).first().removeClass("fa-times");
    $("i", button).first().addClass("fa-list");
  }
}

// Load tunnel
$(tunnelIdentifier).load(
  "https://www.domainelaborde.com/assets/tunnel/ajax/filecontent.php?lang=" +
    tunnel_getLanguage(),

  () => {
    const tunnel = $(this);
    const stepOptions = $(".step[data-code='options']", tunnel).first();

    setTimeout(function () {
      $('[data-toggle="tooltip"]').tooltip();
    }, 100);

    // Sidebar toggle
    $(".sidebar-button", tunnel).on("click", () => {
      tunnel_toggleSidebar();
    });

    $("select.form-control", stepOptions)
      .on("click", function () {
        var p = $(this).parents(".supplements-person").first();
        if (p.length > 0) {
          $(this).data("old-value", $(this).val());
        }
      })
      .on("input", function (e) {
        var p = $(this).parents(".supplements-person").first();
        if (p.length > 0) {
          var stepOptions = $('.step[data-stepcode="options"]', tunnel).first();

          const oldValue = $(this).data("old-value");
          const newValue = $(this).val();

          var total = tunnel_getPeopleAmount();
          if (total > stayData.personmax) {
            $(this).val(oldValue);
            $("#tunnel_peopleAmountWarning span", stepOptions)
              .first()
              .text(stayData.personmax);
            $("#tunnel_peopleAmountWarning", stepOptions).show();
          } else {
            $("#tunnel_peopleAmountWarning", stepOptions).hide();
          }
        }
      });

    $(".form-control", stepOptions).on("change", function () {
      tunnel_updateSidebarPricingDetails();

      // UN SEUL BRANCHEMENT AUTORISÉ
      if ($(this).hasClass("supplement")) {
        var _ids = [8, 9, 10];
        var _id = parseInt($(this).prop("data-supplement-id"));

        if (_ids.includes(_id)) {
          var _val = $(this).val();

          if (_val > 0) {
            for (var i in _ids) {
              if (_ids[i] == _id) continue;
              var _element = $(
                "select#tunnel_supplement_" + _ids[i],
                step_options
              ).first();
              _element.prop("disabled", true);
            }
          } else {
            for (var i in _ids) {
              var _element = $(
                "select#tunnel_supplement_" + _ids[i],
                stepOptions
              ).first();
              _element.prop("disabled", false);
            }
          }
        }
      }

      // CHOIX DE L'EMPLACEMENT
      if ($(this).hasClass("supplement")) {
        var _ids = [115, 112];
        var _id = parseInt($(this).prop("data-supplement-id"));

        if (_ids.includes(_id)) {
          console.log("CHOIX DE L'EMPLACEMENT!", $(this));
          var _val = $(this).val();

          var _field = $("#comment", stepOptions).first();
          var ph_regular = _field.data("placeholder-regular");
          var ph_pitch = _field.data("placeholder-pitch");

          if (_val > 0) {
            //_field.prop('required', true);
            _field.prop("placeholder", ph_pitch);
            $(".isCaptionFor_" + _id).show();
          } else {
            //_field.prop('required', false);
            _field.prop("placeholder", ph_regular);
            $(".isCaptionFor_" + _id).hide();
          }
        }
      }

      if ($(this).parents(".supplements-person").length > 0) {
        tunnel_updateFriendsForm();
      }
    });
    $(".custom-control-input:not(#acceptcgv)", stepOptions).on(
      "change",
      function () {
        tunnel_updateSidebarPricingDetails();
      }
    );

    // Form submit
    $(".step form", tunnel).on("submit", function (e) {
      e.preventDefault();
      if ($("body").data("changestep") !== "true") tunnel_changeStep(true);
    });

    // Stepnav - next
    $(".stepnav .next", tunnel).on("click", function () {
      if ($("body").data("changestep") !== "true") tunnel_changeStep(true);
    });

    // Stepnav - previous
    $(".stepnav .previous", tunnel).on("click", function () {
      if ($("body").data("changestep") !== "true") tunnel_changeStep(false);
    });

    // Steplist click
    $(".steplist li", tunnel).on("click", function (e) {
      e.preventDefault();
      if ($("body").data("changestep") !== "true")
        tunnel_goStep($(this).data("step"));
    });

    // Accept CGV check
    $("input#acceptcgv", tunnel).on("input", function () {
      if ($(this).is(":checked")) {
        $(this).removeClass("is-invalid");
        if ($("body").data("changestep") !== "true") tunnel_changeStep(true);
      }
    });

    // About CVV tooltip
    $(".about-cvv")
      .on("mouseover", function () {
        $(".about-cvv-tooltip", this).show();
      })
      .on("mouseout", function () {
        $(".about-cvv-tooltip", this).hide();
      });
  }
);

// On page load
$(function () {
  console.log(
    "https://www.domainelaborde.com/assets/tunnel/ajax/filecontent.php?lang=" +
      tunnel_getLanguage()
  );

  setTimeout(function () {
    tunnel_loaded(true);
  }, 600);
});
